package com.jiuqi.grid;

import com.jiuqi.util.StreamException;
import com.jiuqi.util.Stream;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Map;
import java.util.List;
import java.io.Serializable;

public class PropList implements Serializable
{
    private List list;
    private Map finder;
    private int bufSize;
    private Key tmpKey;
    
    public PropList(final int bufferSize) {
        this.list = new ArrayList();
        this.finder = new HashMap();
        this.bufSize = bufferSize;
        this.tmpKey = new Key();
    }
    
    public byte[] get(final int index) {
        return this.list.get(index);
    }
    
    public void set(final int index, final byte[] value) {
        this.finder.remove(this.tmpKey.setProperty(this.list.get(index)));
        final byte[] b = this.list.get(index);
        System.arraycopy(value, 0, b, 0, this.bufSize);
        this.finder.put(new Key(b), new Integer(index));
    }
    
    public int indexOf(final byte[] propData) {
        final Integer index = this.finder.get(this.tmpKey.setProperty(propData));
        return (index == null) ? -1 : index;
    }
    
    public int count() {
        return this.list.size();
    }
    
    @Deprecated
    public void setCount(final int value) {
        for (int i = this.list.size(); i < value; ++i) {
            this.list.add(new byte[this.bufSize]);
        }
        for (int i = this.list.size() - 1; i >= value; --i) {
            this.list.remove(i);
        }
    }
    
    public int add(final byte[] propData) {
        final byte[] b = new byte[this.bufSize];
        System.arraycopy(propData, 0, b, 0, this.bufSize);
        final int index = this.list.size();
        this.list.add(b);
        this.finder.put(new Key(b), new Integer(index));
        return index;
    }
    
    private void rebuild() {
        this.finder.clear();
        for (int i = 0; i < this.list.size(); ++i) {
            this.finder.put(new Key(this.list.get(i)), new Integer(i));
        }
    }
    
    public void loadFromStream(final Stream stream, final int size) throws StreamException {
        this.list.clear();
        for (int i = 0; i < size / this.bufSize; ++i) {
            final byte[] b = new byte[this.bufSize];
            stream.readBuffer(b, 0, this.bufSize);
            this.list.add(b);
        }
        this.rebuild();
    }
    
    public void loadFromStreamEx(final Stream stream, final int bufStart, final int bufLen) throws StreamException {
        final int count = (int)(stream.getSize() / bufLen);
        this.setCount(count);
        stream.setPosition(0L);
        for (int i = 0; i < count; ++i) {
            final byte[] buf = this.list.get(i);
            stream.readBuffer(buf, bufStart, bufLen);
        }
        this.rebuild();
    }
    
    public void saveToStream(final Stream stream) throws StreamException {
        for (int i = 0; i < this.list.size(); ++i) {
            final byte[] b = this.list.get(i);
            stream.writeBuffer(b, 0, this.bufSize);
        }
    }
    
    public void saveToStreamEx(final Stream stream, final int bufStart, final int bufLen) throws StreamException {
        for (int i = 0; i < this.list.size(); ++i) {
            final byte[] buf = this.list.get(i);
            stream.writeBuffer(buf, bufStart, bufLen);
        }
    }
    
    public void remove(final int index) {
        this.list.remove(index);
        this.rebuild();
    }
    
    private static final class Key
    {
        private byte[] property;
        private int hash;
        
        public Key() {
        }
        
        public Key(final byte[] property) {
            this.setProperty(property);
        }
        
        public Key setProperty(final byte[] property) {
            this.property = property;
            if (property == null) {
                this.hash = 0;
            }
            else {
                this.hash = 1;
                for (int i = 0; i < property.length; ++i) {
                    this.hash = 31 * this.hash + property[i];
                }
            }
            return this;
        }
        
        @Override
        public int hashCode() {
            return this.hash;
        }
        
        @Override
        public boolean equals(final Object obj) {
            if (this.hashCode() != obj.hashCode() || !(obj instanceof Key)) {
                return false;
            }
            final Key another = (Key)obj;
            if (this.property == null) {
                return another.property == null;
            }
            for (int i = 0; i < this.property.length; ++i) {
                if (this.property[i] != another.property[i]) {
                    return false;
                }
            }
            return true;
        }
        
        @Override
        public String toString() {
            final StringBuffer s = new StringBuffer();
            s.append(this.property).append('(').append(this.hash).append(')');
            return s.toString();
        }
    }
}
